const jwt = require("jsonwebtoken")

function checkPrivilege(permission, privilege) {
    if(Array.isArray(permission)) {
        let has_access = false
        permission.forEach(role_needed => {
            if(Array.isArray(privilege) && privilege.indexOf(role_needed) !== -1) {
                has_access = true
            } else if(typeof privilege[role_needed] !== 'undefined') {
                has_access = true
            }
        })
        return has_access
    }
    else {
        if(Array.isArray(privilege) && privilege.indexOf(permission) !== -1) {
            return true
        }
        else if(typeof privilege[permission] !== 'undefined'){
            return true
        }
    }
    return false
}

const auth = (permission = null) => {
    return (req, res, next) => {
        const authHeader = req.headers['authorization']
        let token = authHeader && authHeader.split(' ')[1]
        if(typeof req.query['access_token'] !== 'undefined' && req.query.access_token !== '') {
            token = req.query.access_token
        }
        if(token === null) {
            return res.status(401).send({ code: 401, message: 'Unauthorized' })
        }
        jwt.verify(token, process.env.JWT_TOKEN, (err, user) => {
            if(err) {
                return res.status(401).send({ code: 401, message: 'Unauthorized' })
            }
            req.context = user

            if(permission !== null && user.agent_type !== 'super_admin') {
                if(checkPrivilege(permission, user.privilege)) {
                    next()
                    return
                }
                return res.status(403).send({ code: 403, message: 'No permission' })
            }

            next()
        })
    }
    
}

module.exports = auth