const newrelic = require('newrelic')
module.exports = async (req, res, functionName) => {
    const context = {
        headers: {
            authorization: req.headers.authorization
        },
        params: req.params,
        query: req.query,
        body: req.body,
        userContext: req.context
    }

    if(typeof newrelic !== 'undefined' && typeof newrelic.addCustomAttributes !== 'undefined' && typeof context.userContext !== 'undefined') {
        newrelic.addCustomAttributes({
            agentId:  context.userContext.agent_id
        })
    }
    const { code, message, data, type } = await functionName(context)
    if(type === 'html') {
        res.writeHeader(code, {"Content-Type": "text/html"})
        res.write(data)
        res.end()
        return res
    }
    else if(type === 'file') {
        const { binary, filename } = data
        res.attachment(filename); 
        return res.send(binary);
    }
    else if(type === 'pdf') {
        const { binary } = data
        res.setHeader('Content-Disposition', 'inline; filename="file.pdf"');
        res.contentType('application/pdf');
        return res.send(binary);
    }
    return res.status(code).send({
        code,
        message,
        data
    })
}
