const redis = require('./redis')

async function manageLimit (name,unique_object, limit, time) {
    const string_object = JSON.stringify(unique_object)
    const base64_hash = await Buffer.from(string_object).toString('base64')
    const redis_key = `limiter_${name}_${base64_hash}`
    return validate(redis_key, { time, limit })
}
async function validate (redis_key, config) {
    const { time, limit } = config
    const unlock =  await redis.lock(redis_key)
    const session = await redis.client.get(redis_key)
    if(session === null) {
        await redis.client.set(redis_key, 0)
        await redis.client.expire(redis_key, time)
    }
    if(session >= limit) {
        await unlock()
        return {
            reach_limit: true,
            session
        }
    }
    await redis.client.incr(redis_key)
    await unlock()
    return {
        reach_limit: false,
        session: false,
    }
}

async function endpoint(req, res, next) {
    let limiterConfig = {
        queryAsKey: true,
        limit: [
            { type: 'rps', limit: 1, time: 1 },
            { type: 'rpm', limit: 5, time: 60 },
            { type: 'rph', limit: null, time: 60 * 60 },
        ]
    }
    const { path, query, headers, config = {} } = req
    limiterConfig = {
        ...limiterConfig,
        ...config.limiter
    }
    const user_ip = (typeof headers['x-forwarded-for'] === 'string'
            && headers['x-forwarded-for'].split(',').shift())
            || req.connection?.remoteAddress
            || req.socket?.remoteAddress
            || req.connection?.socket?.remoteAddress
    let hash_request = user_ip + path
    if(config.limiter.queryAsKey) {
        hash_request +=  JSON.stringify(query)
    }
    const base64_hash = await Buffer.from(hash_request).toString('base64')
    const redis_key = `limiter_${base64_hash}`

    for(let i=0; i<limiterConfig.limit.length; i++) {
        const limit = limiterConfig.limit[i]
        if(limit.limit === null) { continue }
        const detail_redis_key = `${limit.type}_${redis_key}`
        const limit_check = await validate(detail_redis_key, limit)
        if(limit_check.reach_limit) {
            console.info(`Rate limiter block ${user_ip} ${path} ${limit.type} (${limit_check.session})`)
            return res.status(429).send({ code: 429, message: 'Too Many Requests' })
        }
    }
    
    next()
}

exports.manage = manageLimit
exports.endpoint = endpoint